package sun.java2d.pipe;

import java.awt.image.LookupTable;
import java.awt.image.ShortLookupTable;
import java.awt.image.ByteLookupTable;
import java.awt.image.ColorModel;
import java.awt.image.IndexColorModel;
import java.awt.image.Kernel;
import java.awt.image.LookupOp;
import java.awt.image.RescaleOp;
import java.awt.image.ConvolveOp;
import java.awt.image.BufferedImageOp;
import java.awt.image.BufferedImage;
import sun.java2d.SurfaceData;

public class BufferedBufImgOps
{
    public BufferedBufImgOps() {
    }
    
    public static void enableBufImgOp(RenderQueue renderQueue, SurfaceData surfaceData, BufferedImage bufferedImage, BufferedImageOp bufferedImageOp) {
        if ((bufferedImageOp instanceof ConvolveOp)) {
            enableConvolveOp(renderQueue, surfaceData, (ConvolveOp)bufferedImageOp);
        }
        else if ((bufferedImageOp instanceof RescaleOp)) {
            enableRescaleOp(renderQueue, surfaceData, bufferedImage, (RescaleOp)bufferedImageOp);
        }
        else if ((bufferedImageOp instanceof LookupOp)) {
            enableLookupOp(renderQueue, surfaceData, bufferedImage, (LookupOp)bufferedImageOp);
        }
        else {
            throw new InternalError("Unknown BufferedImageOp");
        }
    }
    
    public static void disableBufImgOp(RenderQueue renderQueue, BufferedImageOp bufferedImageOp) {
        if ((bufferedImageOp instanceof ConvolveOp)) {
            disableConvolveOp(renderQueue);
        }
        else if ((bufferedImageOp instanceof RescaleOp)) {
            disableRescaleOp(renderQueue);
        }
        else if ((bufferedImageOp instanceof LookupOp)) {
            disableLookupOp(renderQueue);
        }
        else {
            throw new InternalError("Unknown BufferedImageOp");
        }
    }
    
    public static boolean isConvolveOpValid(ConvolveOp convolveOp) {
        Kernel kernel = convolveOp.getKernel();
        int width = kernel.getWidth();
        int height = kernel.getHeight();
        return (width == 3 && height == 3) || (width == 5 && height == 5);
    }
    
    private static void enableConvolveOp(RenderQueue renderQueue, SurfaceData surfaceData, ConvolveOp convolveOp) {
        int n = convolveOp.getEdgeCondition() == 0;
        Kernel kernel = convolveOp.getKernel();
        int width = kernel.getWidth();
        int height = kernel.getHeight();
        int n2 = 24 + width * height * 4;
        RenderBuffer buffer = renderQueue.getBuffer();
        renderQueue.ensureCapacityAndAlignment(n2, 4);
        buffer.putInt(120);
        buffer.putLong(surfaceData.getNativeOps());
        buffer.putInt(n ? 1 : 0);
        buffer.putInt(width);
        buffer.putInt(height);
        buffer.put(kernel.getKernelData((float[])null));
    }
    
    private static void disableConvolveOp(RenderQueue renderQueue) {
        RenderBuffer buffer = renderQueue.getBuffer();
        renderQueue.ensureCapacity(4);
        buffer.putInt(121);
    }
    
    public static boolean isRescaleOpValid(RescaleOp rescaleOp, BufferedImage bufferedImage) {
        int numFactors = rescaleOp.getNumFactors();
        ColorModel colorModel = bufferedImage.getColorModel();
        if ((colorModel instanceof IndexColorModel)) {
            throw new IllegalArgumentException("Rescaling cannot be performed on an indexed image");
        }
        if (numFactors != 1 && numFactors != colorModel.getNumColorComponents() && numFactors != colorModel.getNumComponents()) {
            throw new IllegalArgumentException("Number of scaling constants does not equal the number of of color or color/alpha  components");
        }
        int type = colorModel.getColorSpace().getType();
        return (type == 5 || type == 6) && numFactors != 2 && numFactors <= 4;
    }
    
    private static void enableRescaleOp(RenderQueue renderQueue, SurfaceData surfaceData, BufferedImage bufferedImage, RescaleOp rescaleOp) {
        ColorModel colorModel = bufferedImage.getColorModel();
        int n = colorModel.hasAlpha() && colorModel.isAlphaPremultiplied();
        int numFactors = rescaleOp.getNumFactors();
        float[] scaleFactors = rescaleOp.getScaleFactors((float[])null);
        float[] offsets = rescaleOp.getOffsets((float[])null);
        float[] array;
        float[] array2;
        if (numFactors == 1) {
            array = new float[4];
            array2 = new float[4];
            for (int j = 0; j < 3; ++j) {
                array[j] = scaleFactors[0];
                array2[j] = offsets[0];
            }
            array[3] = 1.0f;
            array2[3] = 0.0f;
        }
        else if (numFactors == 3) {
            array = new float[4];
            array2 = new float[4];
            for (int i = 0; i < 3; ++i) {
                array[i] = scaleFactors[i];
                array2[i] = offsets[i];
            }
            array[3] = 1.0f;
            array2[3] = 0.0f;
        }
        else {
            array = scaleFactors;
            array2 = offsets;
        }
        if (colorModel.getNumComponents() == 1) {
            int n4 = (1 << colorModel.getComponentSize(0)) - 1;
            for (int l = 0; l < 3; ++l) {
                int n5 = l;
                array2[n5] /= (float)n4;
            }
        }
        else {
            for (int k = 0; k < colorModel.getNumComponents(); ++k) {
                int n2 = (1 << colorModel.getComponentSize(k)) - 1;
                int n3 = k;
                array2[n3] /= (float)n2;
            }
        }
        int n6 = 16 + 4 * 4 * 2;
        RenderBuffer buffer = renderQueue.getBuffer();
        renderQueue.ensureCapacityAndAlignment(n6, 4);
        buffer.putInt(122);
        buffer.putLong(surfaceData.getNativeOps());
        buffer.putInt(n ? 1 : 0);
        buffer.put(array);
        buffer.put(array2);
    }
    
    private static void disableRescaleOp(RenderQueue renderQueue) {
        RenderBuffer buffer = renderQueue.getBuffer();
        renderQueue.ensureCapacity(4);
        buffer.putInt(123);
    }
    
    public static boolean isLookupOpValid(LookupOp lookupOp, BufferedImage obj) {
        LookupTable table = lookupOp.getTable();
        int numComponents = table.getNumComponents();
        ColorModel colorModel = obj.getColorModel();
        if ((colorModel instanceof IndexColorModel)) {
            throw new IllegalArgumentException("LookupOp cannot be performed on an indexed image");
        }
        if (numComponents != 1 && numComponents != colorModel.getNumComponents() && numComponents != colorModel.getNumColorComponents()) {
            throw new IllegalArgumentException("Number of arrays in the  lookup table (" + numComponents + ") is not compatible with the src image: " + obj);
        }
        int type = colorModel.getColorSpace().getType();
        if (type != 5 && type != 6) {
            return false;
        }
        if (numComponents == 2 || numComponents > 4) {
            return false;
        }
        if ((table instanceof ByteLookupTable)) {
            byte[][] table3 = ((ByteLookupTable)table).getTable();
            int j = 1;
            while (j < table3.length) {
                if (table3[j].length > 256 || table3[j].length != table3[j - 1].length) {
                    return false;
                }
                ++j;
            }
        }
        else if ((table instanceof ShortLookupTable)) {
            short[][] table2 = ((ShortLookupTable)table).getTable();
            int i = 1;
            while (i < table2.length) {
                if (table2[i].length > 256 || table2[i].length != table2[i - 1].length) {
                    return false;
                }
                ++i;
            }
        }
        else {
            return false;
        }
        return true;
    }
    
    private static void enableLookupOp(RenderQueue renderQueue, SurfaceData surfaceData, BufferedImage bufferedImage, LookupOp lookupOp) {
        int n = bufferedImage.getColorModel().hasAlpha() && bufferedImage.isAlphaPremultiplied();
        LookupTable table = lookupOp.getTable();
        int numComponents = table.getNumComponents();
        int offset = table.getOffset();
        int n2;
        int n3;
        int n4;
        if ((table instanceof ShortLookupTable)) {
            n2 = ((ShortLookupTable)table).getTable()[0].length;
            n3 = 2;
            n4 = true;
        }
        else {
            n2 = ((ByteLookupTable)table).getTable()[0].length;
            n3 = 1;
            n4 = false;
        }
        int n5 = numComponents * n2 * n3;
        int n6 = n5 + 3 & 0xFFFFFFFC;
        int n7 = n6 - n5;
        int n8 = 32 + n6;
        RenderBuffer buffer = renderQueue.getBuffer();
        renderQueue.ensureCapacityAndAlignment(n8, 4);
        buffer.putInt(124);
        buffer.putLong(surfaceData.getNativeOps());
        buffer.putInt(n ? 1 : 0);
        buffer.putInt(n4 ? 1 : 0);
        buffer.putInt(numComponents);
        buffer.putInt(n2);
        buffer.putInt(offset);
        if (!(n4 == 0)) {
            short[][] table3 = ((ShortLookupTable)table).getTable();
            for (int j = 0; j < numComponents; ++j) {
                buffer.put(table3[j]);
            }
        }
        else {
            byte[][] table2 = ((ByteLookupTable)table).getTable();
            for (int i = 0; i < numComponents; ++i) {
                buffer.put(table2[i]);
            }
        }
        if (n7 != 0) {
            buffer.position((long)(buffer.position() + n7));
        }
    }
    
    private static void disableLookupOp(RenderQueue renderQueue) {
        RenderBuffer buffer = renderQueue.getBuffer();
        renderQueue.ensureCapacity(4);
        buffer.putInt(125);
    }
}
